import fs from 'fs';

export interface StringMap<T> { [key: string]: T; }; // Enables using keys as strings

export interface ESOManifest extends StringMap<string | string[] | boolean | undefined> {
  Title: string
  APIVersion: string
  AddonVersion: string
  Files?: string[]
  Version?: string
  Description?: string
  Author?: string
  SavedVariables?: string
  DependsOn?: string
  OptionalDependsOn?: string
  IsLibrary?: boolean
}

/**
 * Generates a manifest text file Elder Scrolls Online addons.
 * See Manifest type definition for available options.
 * Also see https://wiki.esoui.com/Addon_manifest_(.txt)_format
 * 
 * @param manifestFileName Name of the manifest file to create.
 * @param manifest Data to write to the manifest file.
 * @param fileDependencies Array of strings
 * @param printDisclosure 
 */
export async function generateManifest(manifestFileName: string, manifest: ESOManifest): Promise<string[]> {
  // Ordered set keys that determine which order to print the lines.
  const keys = [
    "Title",
    "APIVersion",
    "AddOnVersion",
    "Author",
    "Description",
    "Version",
    "SavedVariables",
    "DependsOn",
    "OptionalDependsOn",
    "IsLibrary"];
    
  let manifestLines = ["# Manifest generated using eso-manifest. https://www.npmjs.com/package/eso-manifest"];
  keys.forEach(item => {
    if (manifest[item]) {
      manifestLines = manifestLines.concat(`## ${item}: ${manifest[item]}`);
    }
  });

  manifestLines = manifestLines.concat(manifest.Files ? manifest.Files : []);
  manifestLines = manifestLines.concat("\n# This Add-on is not created by, affiliated with, or sponsored by, ZeniMax Media Inc. or its affiliates.\n# The Elder Scrolls® and related logos are registered trademarks or trademarks of ZeniMax Media Inc. in the United States\n# and/or other countries. All rights reserved.");

  // TODO From wiki: https://wiki.esoui.com/Addon_manifest_(.txt)_format
  // To prevent issues, make sure the file is UTF-8 w/o BOM. UTF-8-BOM and ANSI can lead to some different issues during AddOn loading etc.. Also make sure that each line is not longer than 301 bytes. Anything that goes beyond that will be ignored by the game.
  return new Promise<string[]>((resolve, reject) => fs.writeFile(manifestFileName, manifestLines.join('\n'), error => {
    if (error) {
      reject(error);
    } else {
      console.log(`Saved manifest to ${manifestFileName}`);
      resolve(manifestLines);
    }
  }));
}
